/**
 * Copyright (c) Microsoft Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import type { Fixtures, FrameLocator, Locator, Page, Browser, BrowserContext } from '@playwright/test';
import { CommonFixtures, TestChildProcess } from './commonFixtures';
type BaseTestFixtures = CommonFixtures & {
    context: BrowserContext;
};
type BaseWorkerFixtures = {
    headless: boolean;
    browser: Browser;
    browserName: 'chromium' | 'firefox' | 'webkit';
    playwright: typeof import('@playwright/test');
};
export type TraceViewerFixtures = {
    showTraceViewer: (trace: string | undefined, options?: {
        host?: string;
        port?: number;
        stdin?: boolean;
    }) => Promise<TraceViewerPage>;
    runAndTrace: (body: () => Promise<void>, optsOverrides?: Parameters<BrowserContext['tracing']['start']>[0]) => Promise<TraceViewerPage>;
};
declare class TraceViewerPage {
    page: Page;
    process: TestChildProcess;
    actionTitles: Locator;
    actionsTree: Locator;
    callLines: Locator;
    consoleLines: Locator;
    logLines: Locator;
    errorMessages: Locator;
    consoleLineMessages: Locator;
    consoleStacks: Locator;
    networkRequests: Locator;
    metadataTab: Locator;
    snapshotContainer: Locator;
    sourceCodeTab: Locator;
    networkTab: Locator;
    settingsDialog: Locator;
    themeSetting: Locator;
    displayCanvasContentSetting: Locator;
    constructor(page: Page, process: TestChildProcess);
    showAllActions(): Promise<void>;
    stackFrames(options?: {
        selected?: boolean;
    }): any;
    actionIconsText(action: string): any;
    actionIcons(action: string): any;
    expandAction(title: string): Promise<void>;
    selectAction(title: string, ordinal?: number): Promise<void>;
    hoverAction(title: string, ordinal?: number): Promise<void>;
    selectSnapshot(name: string): Promise<void>;
    showErrorsTab(): Promise<void>;
    showConsoleTab(): Promise<void>;
    showSourceTab(): Promise<void>;
    showNetworkTab(): Promise<void>;
    showMetadataTab(): Promise<void>;
    showSettings(): Promise<void>;
    snapshotFrame(actionName: string, ordinal?: number, hasSubframe?: boolean): Promise<FrameLocator>;
}
export declare const traceViewerFixtures: Fixtures<TraceViewerFixtures, {}, BaseTestFixtures, BaseWorkerFixtures>;
export {};
//# sourceMappingURL=traceViewerFixtures.d.ts.map